@@ -304,8 +304,9 @@ def sort_file(
                                 file_output=tmp_file.read_text(encoding=source_file.encoding),
                                 file_path=file_path or source_file.path,
                             )
-                            if ask_to_apply and not ask_whether_to_apply_changes_to_file(
-                                str(source_file.path)
+                            if show_diff or (
+                                ask_to_apply
+                                and not ask_whether_to_apply_changes_to_file(str(source_file.path))
                             ):
                                 return
                         source_file.stream.close()
@@ -395,14 +396,14 @@ def _sort_imports(
             if not line_separator:
                 line_separator = "\n"
         else:
-            if not line_separator:
-                line_separator = line[-1]
+            stripped_line = line.strip()
+            if stripped_line and not line_separator:
+                line_separator = line[len(line.rstrip()) :].replace(" ", "").replace("\t", "")
 
             for file_skip_comment in FILE_SKIP_COMMENTS:
                 if file_skip_comment in line:
                     raise FileSkipComment("Passed in content")
 
-            stripped_line = line.strip()
             if (
                 (index == 0 or (index in (1, 2) and not contains_imports))
                 and stripped_line.startswith("#")
@@ -450,6 +451,9 @@ def _sort_imports(
                     isort_off = True
                 elif stripped_line == "# isort: split":
                     not_imports = True
+                elif stripped_line in config.section_comments and not import_section:
+                    import_section += line
+                    indent = line[: -len(line.lstrip())]
                 elif not (stripped_line or contains_imports):
                     if add_imports and not indent:
                         import_section += line_separator.join(add_imports) + line_separator
