@@ -1,7 +1,7 @@
 import copy
 import itertools
 from functools import partial
-from typing import Iterable, List, Tuple
+from typing import Dict, Iterable, List, Tuple
 
 from isort.format import format_simplified
 
@@ -57,9 +57,6 @@ def sorted_imports(
             from_modules, key=lambda key: sorting.module_key(key, config, section_name=section)
         )
 
-        if config.force_sort_within_sections:
-            copied_comments = copy.deepcopy(parsed.categorized_comments)
-
         section_output: List[str] = []
         if config.from_first:
             section_output = _with_from_imports(
@@ -107,11 +104,20 @@ def sorted_imports(
             )
 
         if config.force_sort_within_sections:
-            # Remove comments
-            section_output = [line for line in section_output if not line.startswith("#")]
+            # collapse comments
+            comments_above = []
+            new_section_output: List[str] = []
+            for line in section_output:
+                if line.startswith("#"):
+                    comments_above.append(line)
+                elif comments_above:
+                    new_section_output.append(_LineWithComments(line, comments_above))
+                    comments_above = []
+                else:
+                    new_section_output.append(line)
 
-            section_output = sorting.naturally(
-                section_output,
+            new_section_output = sorting.naturally(
+                new_section_output,
                 key=partial(
                     sorting.section_key,
                     order_by_type=config.order_by_type,
@@ -121,19 +127,15 @@ def sorted_imports(
                 ),
             )
 
-            # Add comments back
-            all_comments = copied_comments["above"]["from"]
-            all_comments.update(copied_comments["above"]["straight"])
-            comment_indexes = {}
-            for module, comment_list in all_comments.items():
-                for idx, line in enumerate(section_output):
-                    if module in line:
-                        comment_indexes[idx] = comment_list
-            added = 0
-            for idx, comment_list in comment_indexes.items():
-                for comment in comment_list:
-                    section_output.insert(idx + added, comment)
-                    added += 1
+            # uncollapse comments
+            section_output = []
+            for line in new_section_output:
+                comments = getattr(line, "comments", ())
+                if comments:
+                    if new_section_output and config.ensure_newline_before_comments:
+                        section_output.append("")
+                    section_output.extend(comments)
+                section_output.append(str(line))
 
         section_name = section
         no_lines_before = section_name in config.no_lines_before
@@ -176,16 +178,14 @@ def sorted_imports(
 
     if len(formatted_output) > imports_tail:
         next_construct = ""
-        _in_quote: str = ""
         tail = formatted_output[imports_tail:]
 
         for index, line in enumerate(tail):
-            in_quote = _in_quote
-            should_skip, _in_quote, *_ = parse.skip_line(
+            should_skip, in_quote, *_ = parse.skip_line(
                 line,
-                in_quote=_in_quote,
+                in_quote="",
                 index=len(formatted_output),
-                section_comments=parsed.section_comments,
+                section_comments=config.section_comments,
             )
             if not should_skip and line.strip():
                 if (
@@ -196,16 +196,9 @@ def sorted_imports(
                     continue
                 next_construct = line
                 break
-            elif not in_quote:
-                parts = line.split()
-                if (
-                    len(parts) >= 3
-                    and parts[1] == "="
-                    and "'" not in parts[0]
-                    and '"' not in parts[0]
-                ):
-                    next_construct = line
-                    break
+            elif in_quote:
+                next_construct = line
+                break
 
         if config.lines_after_imports != -1:
             formatted_output[imports_tail:0] = ["" for line in range(config.lines_after_imports)]
@@ -283,6 +276,12 @@ def _with_from_imports(
 
         while from_imports:
             comments = parsed.categorized_comments["from"].pop(module, ())
+            above_comments = parsed.categorized_comments["above"]["from"].pop(module, None)
+            if above_comments:
+                if new_section_output and config.ensure_newline_before_comments:
+                    new_section_output.append("")
+                new_section_output.extend(above_comments)
+
             if "*" in from_imports and config.combine_star:
                 import_statement = wrap.line(
                     with_comments(
@@ -338,12 +337,6 @@ def _with_from_imports(
                         )
                     comments = None
             else:
-                above_comments = parsed.categorized_comments["above"]["from"].pop(module, None)
-                if above_comments:
-                    if new_section_output and config.ensure_newline_before_comments:
-                        new_section_output.append("")
-                    new_section_output.extend(above_comments)
-
                 while from_imports and from_imports[0] in as_imports:
                     from_import = from_imports.pop(0)
                     as_imports[from_import] = sorting.naturally(as_imports[from_import])
@@ -474,11 +467,6 @@ def _with_from_imports(
                     import_statement = wrap.line(import_statement, parsed.line_separator, config)
 
             if import_statement:
-                above_comments = parsed.categorized_comments["above"]["from"].pop(module, None)
-                if above_comments:  # pragma: no cover
-                    if new_section_output and config.ensure_newline_before_comments:
-                        new_section_output.append("")
-                    new_section_output.extend(above_comments)
                 new_section_output.append(import_statement)
     return new_section_output
 
@@ -535,3 +523,9 @@ def _normalize_empty_lines(lines: List[str]) -> List[str]:
 
     lines.append("")
     return lines
+
+
+class _LineWithComments(str):
+    def __new__(cls, value, comments):
+        cls.comments = comments
+        return super().__new__(cls, value)  # type: ignore
