@@ -51,9 +51,11 @@ def _normalize_line(raw_line: str) -> Tuple[str, str]:
     return (line, raw_line)
 
 
-def import_type(line: str) -> Optional[str]:
+def import_type(line: str, config: Config = DEFAULT_CONFIG) -> Optional[str]:
     """If the current line is an import line it will return its type (from or straight)"""
-    if "isort:skip" in line or "isort: skip" in line or "NOQA" in line:
+    if config.honor_noqa and line.lower().rstrip().endswith("noqa"):
+        return None
+    elif "isort:skip" in line or "isort: skip" in line:
         return None
     elif line.startswith(("import ", "cimport ")):
         return "straight"
@@ -78,7 +80,7 @@ def _strip_syntax(import_string: str) -> str:
 
 
 def skip_line(
-    line: str, in_quote: str, index: int, section_comments: List[str]
+    line: str, in_quote: str, index: int, section_comments: Tuple[str, ...]
 ) -> Tuple[bool, str]:
     """Determine if a given line should be skipped.
 
@@ -132,7 +134,6 @@ class ParsedContent(NamedTuple):
     original_line_count: int
     line_separator: str
     sections: Any
-    section_comments: List[str]
 
 
 def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedContent:
@@ -141,7 +142,6 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
     in_lines = contents.split(line_separator)
     out_lines = []
     original_line_count = len(in_lines)
-    section_comments = [f"# {heading}" for heading in config.import_headings.values()]
     if config.old_finders:
         finder = FindersManager(config=config).find
     else:
@@ -170,10 +170,10 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
         index += 1
         statement_index = index
         (skipping_line, in_quote) = skip_line(
-            line, in_quote=in_quote, index=index, section_comments=section_comments
+            line, in_quote=in_quote, index=index, section_comments=config.section_comments
         )
 
-        if line in section_comments and not skipping_line:
+        if line in config.section_comments and not skipping_line:
             if import_index == -1:
                 import_index = index - 1
             continue
@@ -195,7 +195,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
             (line.strip() for line in line.split(";")) if ";" in line else (line,)  # type: ignore
         ):
             line, raw_line = _normalize_line(line)
-            type_of_import = import_type(line) or ""
+            type_of_import = import_type(line, config) or ""
             if not type_of_import:
                 out_lines.append(raw_line)
                 continue
@@ -363,7 +363,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                             last = out_lines[-1].rstrip()
                         else:
                             last = ""
-                    if statement_index - 1 == import_index:
+                    if statement_index - 1 == import_index:  # pragma: no cover
                         import_index -= len(
                             categorized_comments["above"]["from"].get(import_from, [])
                         )
@@ -433,5 +433,4 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
         original_line_count=original_line_count,
         line_separator=line_separator,
         sections=config.sections,
-        section_comments=section_comments,
     )
