@@ -206,11 +206,15 @@ class Config(_Config):
         config: Optional[_Config] = None,
         **config_overrides,
     ):
+        self._known_patterns: Optional[List[Tuple[Pattern[str], str]]] = None
+        self._section_comments: Optional[Tuple[str, ...]] = None
+
         if config:
             config_vars = vars(config).copy()
             config_vars.update(config_overrides)
             config_vars["py_version"] = config_vars["py_version"].replace("py", "")
             config_vars.pop("_known_patterns")
+            config_vars.pop("_section_comments")
             super().__init__(**config_vars)  # type: ignore
             return
 
@@ -318,7 +322,6 @@ class Config(_Config):
                 combined_config.pop(f"{IMPORT_HEADING_PREFIX}{import_heading_key}")
             combined_config["import_headings"] = import_headings
 
-        self._known_patterns: Optional[List[Tuple[Pattern[str], str]]] = None
         super().__init__(sources=tuple(sources), **combined_config)  # type: ignore
 
     def is_skipped(self, file_path: Path) -> bool:
@@ -378,6 +381,14 @@ class Config(_Config):
 
         return self._known_patterns
 
+    @property
+    def section_comments(self) -> Tuple[str, ...]:
+        if self._section_comments is not None:
+            return self._section_comments
+
+        self._section_comments = tuple(f"# {heading}" for heading in self.import_headings.values())
+        return self._section_comments
+
     @staticmethod
     def _parse_known_pattern(pattern: str) -> List[str]:
         """Expand pattern if identified as a directory and return found sub packages"""
